<?php
namespace Teknisa\MAN\Controllers;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Teknisa\MAN\Utilities;

class DayPartConsolidado  extends CrudWithParameterBag {
    const dataSourceRel      = 'vnd03100_relDayPartConsolidado',
          dataSourceLoja     = 'vnd03100_relDayPartConsolidado_loja';
    
    protected $parameterBagColumnsRel   = array('DTINI', 'NRORG', 'HRINI', 'CDLOJA', 'HRFIN', 'CDFILIAL', 'DTFIN'),
              $parameterBagColumnsLoja  = array('NRORG', 'CDFILIAL'),
              $utilService;
             
              
    public function __construct ($crudService, $parameterBag, $entityManager, Utilities\Util $utilService, Manager $dataSourceManager) {
        $this->crudService               = $crudService;
        $this->parameterBag              = $parameterBag;
        $this->entityManager             = $entityManager;
        $this->utilService               = $utilService;
        $this->dataSourceManager         = $dataSourceManager;
    }
    
    public function findRelConsolidado(DTO\Request\Filter $request, DTO\Response $response){
        $resposta = [];
        $dataSourceName       = $this->dataSourceName = 'vnd03100_relDayPartConsolidado';
        $this->parameterBagColumns = $this->parameterBagColumnsRel;
        $conditions           = $this->utilService->getParamsAsArray($request);
        if(isset($conditions['CDFILIAL']) && !empty($conditions['CDFILIAL'])){
            $dataSourceFilter     = DayPartConsolidado::prepareParameters($conditions);
            $newRequest           = new DTO\Request\Filter(
                new FilterCriteria(
                    $dataSourceName,
                    $dataSourceFilter,
                    $request->getFilterCriteria()->getPage()
                ),
                $request->getMethod(),
                $request->getRoutePath(),
                $request->getUserId()
            );
            $this->parameterBagColumns = $this->parameterBagColumnsRel;
            $newResponse = new DTO\Response();
            parent::find($newRequest, $newResponse);
            $rows = $newResponse->getDataSets()[0]->getRows();
            if(isset($rows)){
                $resposta = DayPartConsolidado::manipulateData($rows, $conditions);
                $response->addDataSet(new DataSet('vnd03100_relDayPartConsolidado', $resposta));
            } else {
                $resposta = array();
                $response->addDataSet(new DataSet('vnd03100_relDayPartConsolidado', $resposta));
            }
        }
        return $resposta;
    }
    
    private function prepareParameters($conditions){
            $FILIAL     = join('_', $conditions['CDFILIAL']);
            $DTINI      = $conditions['DTPERIODO'][0];
            $DTFIM      = $conditions['DTPERIODO'][1];
            //Remove os '%' dos campos de hora e loja
            $FILTROLOJA = str_replace("%", "", $conditions['FILILOJA']);
            $LOJA       = str_replace(',', '_', $FILTROLOJA);
            //$LOJAFILTRO = join('_', 
            //Concatena os códigos de loja com underscore ao invés de virgula
            $HRINI      = str_replace("%", "", $conditions['HRINI']);
            $HRFIN      = str_replace("%", "", $conditions['HRFIN']);
            $parameters = array(
                'DTINI'    => $DTINI,
                'DTFIN'    => $DTFIM,
                'CDFILIAL' => $FILIAL,
                'CDLOJA'   => $LOJA,
                'HRFIN'    => $HRFIN,
                'HRINI'    => $HRINI
            );
            $dataSourceFilter = DayPartConsolidado::createDataSourceFilter($parameters, $conditions);
            return $dataSourceFilter;
    }    
    
    private function createDataSourceFilter($parameters, $conditions){
    //Cria o dataSourceFilter para ser usado pelo crud com os parâmetros enviados pela função prepareParameters
        $dataSourceFilter = array();
        $dataSourceFilter[0] = array(
            "columnName" => 'DTINI',
            "operator"   => "=",
            "value"      => $parameters['DTINI']
        ); 
        $dataSourceFilter[1] = array(
            "columnName" => 'NRORG',
            "operator"   => "=",
            "value"      => $conditions['NRORG']
        );
        $dataSourceFilter[2] = array(
            "columnName" => 'HRINI',
            "operator"   => "=",
            "value"      => $parameters['HRINI']
        );
        $dataSourceFilter[3] = array(
            "columnName" => 'CDLOJA',
            "operator"   => "=",
            "value"      => $parameters['CDLOJA']
        );
        $dataSourceFilter[4] = array(
            "columnName" => 'HRFIN',
            "operator"   => "=",
            "value"      => $parameters['HRFIN']
        );
        $dataSourceFilter[5] = array(
            "columnName" => 'CDFILIAL',
            "operator"   => "=",
            "value"      => $parameters['CDFILIAL']
        );
        
        $dataSourceFilter[6] = array(
            "columnName" => 'DTFIN',
            "operator"   => "=",
            "value"      => $parameters['DTFIN']
        );
        return $dataSourceFilter;
    }
    
    private function manipulateData($rows, $conditions){
        $FILIAL     = join('_', $conditions['CDFILIAL']);
        $DTINICIO   = $conditions['DTPERIODO'][0];
        $DTFIM      = $conditions['DTPERIODO'][1];
        $NRORG      = $conditions['NRORG'];
        $LOJA       = $conditions['CDLOJA'];
        $parameters = array(
            'DATAINI'    => $DTINICIO,
            'DATAFIN'    => $DTFIM,
            'CDFILIAL'   => $FILIAL,
            'NRORG'      => $NRORG
        );
        $VALOR_TOTAL = $this->entityManager->getConnection()->executeQuery(self::GET_VALOR_TOTAL, $parameters)->fetchAll();
        $resposta    = array();
        foreach($rows as $aux){
            //$aux['NMLOJA'] = $NOMELOJA[0]['NMLOJA'];
            $aux['PORCTOT'] = $aux['PORCTOT'];
            //TICKETMEDIO = VRVENDAS / NRCUPONS
            $aux['TICKETMEDIO'] = $aux['VRVENDAS'] / $aux['NRCUPONS'];
            //ITENSPORTICKET = QNTPRODUTOS / NRCUPONS
            $aux['ITENSPORTICKET'] = $aux['QTDE'] / $aux['NRCUPONS'];
            //PERÍODO sendo concatenado com as horas de ínicio e fim do day part respectivo 
            $aux['PERIODO'] = $aux['PERIODO'].' - '.$aux['HRINI1DAYPART'].' às '.$aux['HRFIN1DAYPART'];
            //CALCULO DE FATURAMENTO valor de vendas retornado da query['VRVENDAS'] + Valor total adquirido através da query GET_VALOR_TOTAL
            if(!empty($VALOR_TOTAL)){
                foreach($VALOR_TOTAL as $positions)
                    if($aux['CDFILIAL'] == $positions['CDFILIAL']){
                        if(!empty($positions['VALOR_TOTAL'])){
                            $aux['FATURAMENTO'] =  floatval($aux['VALOR_VENDIDO']) + floatval($positions['VALOR_TOTAL'] + floatval($aux['VRDESCONTO'])) ;
                        } else {
                            $aux['FATURAMENTO'] =  floatval($aux['VALOR_VENDIDO']) + floatval($aux['VRDESCONTO']);
                        }
                }
            }else {
                $aux['FATURAMENTO'] =  floatval($aux['VALOR_VENDIDO']) + floatval($aux['VRDESCONTO']);
            }
            $aux['PORCTOTFIL']  =  $aux['PORCTOTFIL'];
            array_push($resposta, $aux);
        }
        return $resposta;
    }
    
     const GET_VALOR_TOTAL = "SELECT V.CDFILIAL, NVL(SUM(NVL(IT.VRACRITVEND,0))- SUM(NVL(V.VRREPIQUEVENDA,0)), 0) AS VALOR_TOTAL
                            FROM ITEMVENDA IT,
                                LOJA L,
                                VENDA V
                            WHERE IT.CDPRODUTO                = L.CDPRODTAXAENTR
                            AND IT.NRORG                      = L.NRORG
                            AND V.NRSEQVENDA                  = IT.NRSEQVENDA
                            AND V.CDCAIXA                     = IT.CDCAIXA
                            AND V.CDFILIAL                    = IT.CDFILIAL
                            AND V.NRORG                       = IT.NRORG
                            AND L.CDLOJA                      = V.CDLOJA
                            AND L.CDFILIAL                    = V.CDFILIAL
                            AND L.NRORG                       = V.NRORG
                            AND INSTR(:CDFILIAL, V.CDFILIAL)  > 0
                            AND L.IDTRATTAXAENTR              = 'P'
                            AND SUBSTR(V.IDORIGEMVENDA, 0, 3) = 'DLV'
                            AND IT.NRORG                      = :NRORG
                            AND V.IDSITUVENDA                 = 'O'
                            AND V.DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN
                            GROUP BY V.CDFILIAL,
                            SUBSTR(V.IDORIGEMVENDA, 0, 3)";
                            
    const GET_NMLOJA = "SELECT NMLOJA, CDLOJA FROM LOJA WHERE CDLOJA = :CDLOJA AND CDFILIAL = :CDFILIAL";
}