<?php
namespace Teknisa\MAN\Factories;

use Doctrine\DBAL\Connection;

class Ger96500MonitoramentoFechamentoCupomFactory {

    protected $connection;

    public function __construct(Connection $connection) {
        $this->connection = $connection;
    }

    /*
     * Função para buscar os cupons das filiais parametrizadas em um
     * determinado dia.
     */
    public function getCuponsPrincipal($params) {
        $sql = <<<SQL
        SELECT NVL(D.IDSTATUS, 'F') AS IDSTATUS,
  F.CDFILIAL,
  D.CDCAIXA,
  '<b class="boldText">' || E.CDEMPRESA || '</b> - ' || E.NMRAZSOCEMPR || ': <b class="boldText">' || F.CDFILIAL || '</b> - ' || F.NMFILIAL AS DESCEMPRESA,
  NVL(TO_CHAR(D.DTREDUCAOZ, 'DD/MM/YYYY'), :DTREDUCAOZ) AS DTREDUCAOZ,
  F.NRORG
  FROM
  (SELECT RZ.CDFILIAL,
    RZ.CDCAIXA,
    RZ.DTREDUCAOZ,
    DECODE(TU.DTFECHCAIX, NULL, 'M', RZ.IDSTATUS) AS IDSTATUS,
    RZ.NRORG
  FROM REDUCAOZ RZ,
    PARAVEND P,
    TURCAIXA TU
  WHERE ((INSTR(:CDFILIAL, RZ.CDFILIAL)>0)
  OR (:CDFILIAL                        = 'T'))
  AND RZ.DTREDUCAOZ                    = :DTREDUCAOZ
  AND RZ.NRORG                         = :NRORG
  AND RZ.CDFILIAL                      = TU.CDFILIAL
  AND RZ.CDCAIXA                       = TU.CDCAIXA
  AND RZ.NRORG                         = TU.NRORG
  AND TRUNC(RZ.DTREDUCAOZ)             = TRUNC(TU.DTABERCAIX)
  AND RZ.VRVENBRUREDZ                 <> 0
  AND RZ.VRVENLIQREDZ                 <> 0
  AND RZ.CDFILIAL                      = P.CDFILIAL
  AND RZ.NRORG                         = P.NRORG
  AND P.IDFECHACUPOM                   = 'S'
  UNION ALL
  SELECT VE.CDFILIAL,
    VE.CDCAIXA,
    VE.DTREDUCAOZ,
    VE.IDSTATUS,
    VE.NRORG
  FROM
    (SELECT SUM(I.QTPRODVEND*(I.VRUNITVEND+NVL(I.VRUNITVENDCL,0)))                                            AS VRVENBRUREDZ,
      SUM((I.QTPRODVEND     *(I.VRUNITVEND+NVL(I.VRUNITVENDCL,0)))-NVL(I.VRDESITVEND,0)+NVL(I.VRACRITVEND,0)) AS VRVENLIQREDZ,
      V.CDFILIAL,
      V.CDCAIXA,
      TRUNC(V.DTENTRVENDA)                                AS DTREDUCAOZ,
      DECODE(TU.DTFECHCAIX, NULL, 'M', V.IDAPROVACAONFCE) AS IDSTATUS,
      V.NRORG
    FROM VENDA V
    JOIN ITEMVENDA I
    ON V.CDFILIAL    = I.CDFILIAL
    AND V.CDCAIXA    = I.CDCAIXA
    AND V.NRSEQVENDA = I.NRSEQVENDA
    AND V.NRORG      = I.NRORG
    JOIN PARAVEND P
    ON V.CDFILIAL = P.CDFILIAL
    AND V.NRORG   = P.NRORG
    JOIN TURCAIXA TU
    ON V.CDFILIALTUR       = TU.CDFILIAL
    AND V.CDCAIXATUR       = TU.CDCAIXA
    AND V.NRORG            = TU.NRORG
    AND TRUNC(V.DTABERTUR) = TRUNC(TU.DTABERCAIX)
    WHERE V.IDTPEMISVEND  IN ('N','S')
    AND ((INSTR(:CDFILIAL, '_'
      || V.CDFILIAL
      || '_')          >0)
    OR (:CDFILIAL      = 'T'))
    AND V.DTENTRVENDA  = :DTREDUCAOZ
    AND V.NRORG        = :NRORG
    AND V.IDTPAMBNFCE  = '1'
    AND V.IDSTATUSNFCE = 'A'
    AND V.IDSITUVENDA  = 'O'
    AND P.IDFECHACUPOM = 'S'
    GROUP BY V.CDFILIAL,
      V.CDCAIXA,
      TRUNC(V.DTENTRVENDA),
      DECODE(TU.DTFECHCAIX, NULL, 'M', V.IDAPROVACAONFCE),
      V.NRORG
    ) VE
  WHERE VE.VRVENBRUREDZ <> 0
  AND VE.VRVENLIQREDZ   <> 0
  ) D,
  PARAVEND P,
  FILIAL F,
  EMPRESA E
WHERE D.CDFILIAL(+) = F.CDFILIAL
AND D.NRORG(+)      = F.NRORG
AND F.CDFILIAL      = P.CDFILIAL
AND F.NRORG         = P.NRORG
AND F.CDEMPRESA     = E.CDEMPRESA
AND F.NRORG         = E.NRORG
AND F.NRORG         = :NRORG
AND P.IDFECHACUPOM  = 'S'
AND ((INSTR(:CDFILIAL, '_'
  || F.CDFILIAL
  || '_')     >0)
OR (:CDFILIAL = 'T'))
ORDER BY E.CDEMPRESA,
  F.CDFILIAL
SQL;

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Retorna todos os dados obtidos.
        return $statement->fetchAll();
    }

    /*
     * Função para buscar os cupons das filiais parametrizadas em um
     * determinado dia.
     */
    public function getCuponsPorCaixa($params) {
        $sql = "SELECT D.IDSTATUS, D.CDFILIAL, D.CDCAIXA, C.NMCAIXA, D.IDORIGEMREDZ, D.IDTPAPROV, D.OPERADORLIBE, O.CDOPERADOR || ' - ' || O.NMOPERADOR AS OPERADORAPROV, TO_CHAR(D.DTREDUCAOZ, 'DD/MM/YYYY') AS DTREDUCAOZ, TO_CHAR(D.DTFECHCUPOM,'DD/MM/YYYY HH24:MI:SS') AS DTFECHCUPOM, D.NRORG FROM(SELECT DECODE(TU.DTFECHCAIX, NULL, 'M', RZ.IDSTATUS) AS IDSTATUS, RZ.CDFILIAL, RZ.CDCAIXA, RZ.DTREDUCAOZ, RZ.IDORIGEMREDZ, RZ.CDOPERFECHCUPOM, RZ.DTFECHCUPOM, RZ.IDTPAPROV, O.CDOPERADOR ||' - ' ||O.NMOPERADOR AS OPERADORLIBE, RZ.NRORG FROM REDUCAOZ RZ, PARAVEND P, TURCAIXA TU, OPERADOR O WHERE RZ.CDFILIAL = :CDFILIAL AND RZ.DTREDUCAOZ = :DTREDUCAOZ AND RZ.NRORG = :NRORG AND RZ.CDFILIAL = TU.CDFILIAL AND RZ.CDCAIXA = TU.CDCAIXA AND RZ.NRORG = TU.NRORG AND TRUNC(RZ.DTREDUCAOZ) = TRUNC(TU.DTABERCAIX) AND RZ.VRVENBRUREDZ <> 0 AND RZ.VRVENLIQREDZ <> 0 AND RZ.CDFILIAL = P.CDFILIAL AND RZ.NRORG = P.NRORG AND P.IDFECHACUPOM = 'S' AND RZ.CDOPERLIBERCUPOM = O.CDOPERADOR(+) AND RZ.NRORG = O.NRORG(+) UNION ALL SELECT VE.IDSTATUS, VE.CDFILIAL, VE.CDCAIXA, VE.DTREDUCAOZ, VE.IDORIGEMREDZ, VE.CDOPERFECHCUPOM, VE.DTFECHCUPOM, VE.IDTPAPROV, VE.OPERADORLIBE, VE.NRORG FROM (SELECT SUM(I.QTPRODVEND*(I.VRUNITVEND+NVL(I.VRUNITVENDCL,0))) AS VRVENBRUREDZ, SUM((I.QTPRODVEND *(I.VRUNITVEND+NVL(I.VRUNITVENDCL,0)))-NVL(I.VRDESITVEND,0)+NVL(I.VRACRITVEND,0)) AS VRVENLIQREDZ, V.CDFILIAL, V.CDCAIXA, TRUNC(V.DTENTRVENDA) AS DTREDUCAOZ, DECODE(TU.DTFECHCAIX, NULL, 'M', V.IDAPROVACAONFCE) AS IDSTATUS, DECODE(V.IDTPEMISVEND,'N','NFCE','S','SAT') AS IDORIGEMREDZ, V.CDOPERFECHCUPOM, V.DTFECHCUPOM, V.IDTPAPROV, O.CDOPERADOR ||' - ' ||O.NMOPERADOR AS OPERADORLIBE, V.NRORG FROM VENDA V JOIN ITEMVENDA I ON V.CDFILIAL = I.CDFILIAL AND V.CDCAIXA = I.CDCAIXA AND V.NRSEQVENDA = I.NRSEQVENDA AND V.NRORG = I.NRORG JOIN PARAVEND P ON V.CDFILIAL = P.CDFILIAL AND V.NRORG = P.NRORG JOIN TURCAIXA TU ON V.CDFILIALTUR = TU.CDFILIAL AND V.CDCAIXATUR = TU.CDCAIXA AND V.NRORG = TU.NRORG AND TRUNC(V.DTABERTUR) = TRUNC(TU.DTABERCAIX) LEFT JOIN OPERADOR O ON V.CDOPERLIBERCUPOM = O.CDOPERADOR AND V.NRORG = O.NRORG WHERE V.IDTPEMISVEND IN ('N','S') AND V.CDFILIAL = :CDFILIAL AND V.DTENTRVENDA = :DTREDUCAOZ AND V.NRORG = :NRORG AND V.IDTPAMBNFCE = '1' AND V.IDSTATUSNFCE = 'A' AND V.IDSITUVENDA = 'O' AND P.IDFECHACUPOM = 'S' GROUP BY V.CDFILIAL, V.CDCAIXA, TRUNC(V.DTENTRVENDA), DECODE(TU.DTFECHCAIX, NULL, 'M', V.IDAPROVACAONFCE), DECODE(V.IDTPEMISVEND,'N','NFCE','S','SAT'), V.CDOPERFECHCUPOM, V.DTFECHCUPOM, V.IDTPAPROV, O.CDOPERADOR ||' - ' ||O.NMOPERADOR, V.NRORG) VE WHERE VE.VRVENBRUREDZ <> 0 AND VE.VRVENLIQREDZ <> 0 ) D, PARAVEND P, CAIXA C, OPERADOR O WHERE D.CDFILIAL = P.CDFILIAL AND D.NRORG = P.NRORG AND D.CDFILIAL = C.CDFILIAL AND D.CDCAIXA = C.CDCAIXA AND D.NRORG = C.NRORG AND D.CDOPERFECHCUPOM = O.CDOPERADOR(+) AND D.NRORG = O.NRORG(+) AND D.NRORG = :NRORG AND P.IDFECHACUPOM = 'S' AND D.CDFILIAL = :CDFILIAL ORDER BY D.CDCAIXA, D.DTFECHCUPOM";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Retorna todos os dados obtidos.
        return $statement->fetchAll();
    }

    /*
     * Função para buscar os erros de pré-validação previamente armazenados
     * de um cupom pendente.
     */
    public function getErrosPreValidacao($params) {
        $sql = "SELECT P.CDFILIAL, P.CDCAIXA, P.DTREDUCAOZ, P.NRSEQERRO, P.IDTPVALID, P.DSERRONFPRE, P.DSAJUDA, P.NRORG FROM PREVALIDCUPOM P WHERE P.CDFILIAL = :CDFILIAL AND P.CDCAIXA = :CDCAIXA AND P.DTREDUCAOZ = :DTREDUCAOZ AND P.NRORG = :NRORG ORDER BY P.IDTPVALID DESC, P.NRSEQERRO";

        $statement = $this->connection->prepare($sql);

        // Executa o statement com os parâmetros passados.
        $statement->execute($params);

        // Retorna todos os dados obtidos.
        return $statement->fetchAll();
    }

}